#include <math.h>

#include "waves.h"

static double wave0(double ang)
{
	// Normal sine
	
	return sin(ang);
}

static double wave1(double ang)
{
	// Negative part cropped
	
	register double tmp = sin(ang);
	return tmp > 0 ? tmp : 0;
}

static double wave2(double ang)
{
	// Rectified (negative made positive)
	
	register double tmp = sin(ang);
	return tmp >= 0 ? tmp : -tmp;
}

static double wave3(double ang)
{
	// Rectified (negative made positive)
	
	register double tmp = sin(ang);
	if(tmp <= 0.0)return 0.0;
	
	// Only the first quarter of each sine wave
	
	register double cosi = cos(ang);
	if(cosi <= 0.0)return 0.0;
	
	return tmp;
}

static double wave4(double ang)
{
	// Sine, even periods only
	
	register double tmp = sin(ang*2);
	if(tmp <= 0.0)return 0.0;
	tmp = sin(ang);
	return tmp;
}

static double wave5(double ang)
{
	// Abs-sine, even periods only
	
	register double tmp = sin(ang*2);
	if(tmp <= 0.0)return 0.0;

	tmp = sin(ang);
	return tmp >= 0 ? tmp : -tmp;
}

static double wave6(double ang)
{
	// Square
	
	register double tmp = sin(ang);
	
	return tmp > 0 ? 1 : -1;
}

static double wave7(double ang)
{
	// Derived square

	register double tmp = sin(ang*2);
	if(tmp <= 0.0)return 0.0;

	tmp = tan(ang);
	if(tmp > 1)return 1;
	if(tmp < -1)return -1;
	return tmp;
}

double (*waves[8])(double) =
{ wave0, wave1, wave2, wave3,
  wave4, wave5, wave6, wave7 };
